package com.example.wc24h;

import android.app.Dialog;
import android.content.Context;
import android.content.SharedPreferences;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Button;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.RadioButton;

import java.io.File;
import java.sql.Date;
import java.text.DateFormat;

class ProfilesDialog extends Dialog
{
    private MainActivity        mContext        = null;
    private static final int    profileCount    = 4;
    private RadioButton[]       profileButtons  = new RadioButton[profileCount];
    private int                 selectedProfile = 0;
    ProfileSelected             profileSelectedListener;

    ProfilesDialog(MainActivity a)
    {
        super(a, R.style.dialog_custom_style);

        mContext = a;
        setContentView(R.layout.profiles_dialog);
        Window window  = getWindow();
        if (window != null)
        {
            getWindow().setSoftInputMode(WindowManager.LayoutParams.SOFT_INPUT_ADJUST_PAN);
        }

        SharedPreferences prefs = mContext.getSharedPreferences("com.example.wc24h", Context.MODE_PRIVATE);
        selectedProfile = prefs.getInt("com.example.wc24h.selected_profile", 0);

        profileSelectedListener = new ProfileSelected();
        File dir = MainActivity.getSettingsDirFile(mContext);
        configureProfileRow(1, R.id.profile1, dir);
        configureProfileRow(2, R.id.profile2, dir);
        configureProfileRow(3, R.id.profile3, dir);
        configureProfileRow(4, R.id.profile4, dir);

        Button cancel = (Button)findViewById(R.id.cancel_button);
        cancel.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View view)
            {
                ProfilesDialog.this.dismiss();
            }
        });
        Button load = (Button)findViewById(R.id.load_button);
        load.setOnClickListener(new View.OnClickListener()
        {
            @Override
            public void onClick(View view)
            {
                SharedPreferences prefs = mContext.getSharedPreferences("com.example.wc24h", Context.MODE_PRIVATE);
                prefs.edit().putInt("com.example.wc24h.selected_profile", selectedProfile).apply();
                mContext.profileChanged(false);
                ProfilesDialog.this.dismiss();
            }
        });
        selectProfile(selectedProfile);
    }

    private void configureProfileRow(final int profile, int container, final File dir) {
        final int idx = profile - 1;
        DateFormat df = DateFormat.getDateInstance(DateFormat.SHORT);
        LinearLayout l = (LinearLayout)findViewById(container);
        profileButtons[idx] = (RadioButton)l.findViewById(R.id.radioButton);
        profileButtons[idx].setOnClickListener(profileSelectedListener);
        File f = new File(dir, MainActivity.getProfileFileName(idx));
        String descr = "";
        if (f.exists()) {
            Date d = new Date(f.lastModified());
            descr = df.format(d) + " ";
            WC24hSettings tmp = new WC24hSettings();
            tmp.loadData(dir, MainActivity.getProfileFileName(idx));
            descr += tmp.description;
        }
        profileButtons[idx].setText(profile + ": " + descr);

        ImageButton btn = (ImageButton)l.findViewById(R.id.edit_btn);
        btn.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                ProfileDialog dlg = new ProfileDialog(mContext, dir, idx);
                dlg.show();
            }
        });
    }

    private void selectProfile(int idx)
    {
        for (int i = 0; i < profileButtons.length; i++)
        {
            profileButtons[i].setChecked(i == idx);
            if (i == idx)
            {
                File dir = MainActivity.getSettingsDirFile(mContext);
                File f = new File(dir, MainActivity.getProfileFileName(i));
                if (f.exists()) {
                    WC24hSettings tmp = new WC24hSettings();
                    tmp.loadData(dir, MainActivity.getProfileFileName(i));
                }
            }
        }
    }

    @Override
    protected void onStart()
    {
        super.onStart();
    }

    @Override
    protected void onStop()
    {
        super.onStop();
    }

    private class ProfileSelected implements View.OnClickListener
    {

        @Override
        public void onClick(View view)
        {
            for (int i = 0; i < profileButtons.length; i++)
            {
                if (view == profileButtons[i])
                {
                    profileButtons[i].setChecked(true);
                    selectedProfile = i;
                    selectProfile(i);
                }
                else
                {
                    profileButtons[i].setChecked(false);
                }
            }
        }
    }
}